using System.Collections.Generic;
using SimpleJSON;
public class ActionItem
{
    public string Name { get; set; }
    public string Action { get; set; }
    public string Group { get; set; }
    public string When { get; set; }
    public string Effect { get; set; }
    public string Context { get; set; }

    public string Location { get; set; }

    public string Wearing { get; set; }

    public bool Deferred { get; set; }

    public string Layer { get; set; }

    public bool TransitionAction { get; set; }

    public bool PlayTimelineAnim { get; set; }

    public EventTrigger Trigger { get; set; }

    public List<string> LinkedActions { get; set; }

    public ActionItem(string newName, ActionItem actionItem)
    {
        Name = newName;
        Action = ActionNameUtils.FunctionFormatActionName(newName);
        Group = actionItem.Group;
        When = actionItem.When;
        Effect = actionItem.Effect;
        Context = actionItem.Context;
        Location = actionItem.Location;
        Wearing = actionItem.Wearing;
        Deferred = actionItem.Deferred;
        Layer = actionItem.Layer;
        TransitionAction = actionItem.TransitionAction;
        PlayTimelineAnim = actionItem.PlayTimelineAnim;
        Trigger = new EventTrigger(actionItem.Trigger);
        LinkedActions = new List<string>(actionItem.LinkedActions);
    }

    public ActionItem(string name, string action, string when, MVRScript script)
    {
        Name = name;
        Action = action;
        When = when;
        Group = "";
        Effect = "";
        Context = "";
        Location = "";
        Wearing = "";
        Layer = "";
        PlayTimelineAnim = true;
        Trigger = new EventTrigger(script, "Trigger");
        LinkedActions = new List<string>();
    }

    public ActionItem(JSONClass actionJSON, MVRScript script)
    {
        Action = actionJSON["Action"];
        Group = actionJSON["Group"];
        if (Group == null)
        {
            Group = "";
        }
        Name = actionJSON["Name"];
        if (Name == null)
        {
            Name = Action;
        }
        Layer = actionJSON["Layer"];
        if (Layer == null)
        {
            Layer = "";
        }
        When = actionJSON["When"];
        Effect = actionJSON["Effect"];
        Context = actionJSON["Context"];
        Location = actionJSON["Location"];
        Wearing = actionJSON["Wearing"];
        Deferred = actionJSON["Deferred"].AsBool;
        TransitionAction = actionJSON["TransitionAction"].AsBool;
        PlayTimelineAnim = actionJSON["PlayTimelineAnim"].AsBool;
        JSONClass triggerJSON = actionJSON["Trigger"].AsObject;
        Trigger = new EventTrigger(script, "Trigger");
        Trigger.RestoreFromJSON(triggerJSON);

        JSONArray linkedActionsArray = actionJSON["LinkedActions"].AsArray;

        LinkedActions = new List<string>();
        foreach (JSONNode linkedActionJSON in linkedActionsArray)
        {
            LinkedActions.Add(linkedActionJSON.Value);
        }
    }

    public JSONClass GetJSON(string subScenePrefix)
    {
        JSONClass actionJSON = new JSONClass();
        actionJSON["Name"] = Name;
        actionJSON["Action"] = Action;
        actionJSON["Group"] = Group;
        actionJSON["When"] = When;
        actionJSON["Effect"] = Effect;
        actionJSON["Context"] = Context;
        actionJSON["Location"] = Location;
        actionJSON["Wearing"] = Wearing;
        actionJSON["Deferred"].AsBool = Deferred;
        actionJSON["Layer"] = Layer;
        actionJSON["TransitionAction"].AsBool = TransitionAction;
        actionJSON["PlayTimelineAnim"].AsBool = PlayTimelineAnim;
        actionJSON["Trigger"] = Trigger.GetJSON(subScenePrefix);

        JSONArray linkedActionsArray = new JSONArray();
        foreach (string linkedAction in LinkedActions)
        {
            linkedActionsArray.Add("", new JSONData(linkedAction));
        }
        actionJSON["LinkedActions"] = linkedActionsArray;

        return actionJSON;
    }
}
