using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using SimpleJSON;
using GPUTools.Common.Scripts.Utils;
using UnityEngine.UI;
using System.Reflection;

public class Persistence : MVRScript
{
    public Dictionary<string, ActionItem> actionsDictionary = new Dictionary<string, ActionItem>();

    public Context currentContext = new Context("", "", "");

    public ActionItem GetItemByName(string name)
    {
        if (actionsDictionary.Count > 0)
        {
            ActionItem actionItem;
            if (actionsDictionary.TryGetValue(name, out actionItem))
            {
                return actionItem;
            }

            SuperController.LogError("Action not found by name = " + name);
        }
        return null;
    }
    public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
    {
        JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
        if (includePhysical || forceStore)
        {
            needsStore = true;

            JSONArray actionsArray = new JSONArray();
            foreach (var kvp in actionsDictionary)
            {
                actionsArray.Add(kvp.Value.GetJSON(base.subScenePrefix));
            }

            jc["Actions"] = actionsArray;
            jc["CurrentContext"] = currentContext.GetJSON();
        }

        return jc;
    }

    public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
    {
        base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
        if (!base.physicalLocked && restorePhysical)
        {
            JSONArray actionsArray = jc["Actions"].AsArray;
            foreach (JSONClass actionJSON in actionsArray)
            {
                ActionItem actionItem = new ActionItem(actionJSON, this);
                actionsDictionary[actionItem.Name] = actionItem;
            }
            JSONClass contextJSON = jc["CurrentContext"].AsObject;
            if (currentContext != null)
            {
                currentContext = new Context(contextJSON);
            }
        }
    }

    public override bool ShouldIgnore() => true;
}
