using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class NewLinkedActionsUI : UIPart
{
    private JSONStorableStringChooser actionsChooser;
    private Dictionary<string, ActionItem> actionsDictionary;
    private UIPart linkedActionsUI;
    public GameObject backDrop;
    public GameObject backDropLock;
    public RectTransform scrollContent;
    private RectTransform scrollContentLeft;
    private RectTransform scrollContentRight;
    private Dictionary<string, bool> groupExpanded = new Dictionary<string, bool>();
    private float lastClickTime = 0f;
    private const float doubleClickThreshold = 0.2f;
    private string pendingSingleClickName;
    JSONStorableString searchField;

    public NewLinkedActionsUI(ActionListManager script, JSONStorableStringChooser actionChooser, Dictionary<string, ActionItem> actionsDictionary, UIPart linkedActionsUI) : base(script)
    {
        this.actionsChooser = actionChooser;
        this.actionsDictionary = actionsDictionary;
        this.linkedActionsUI = linkedActionsUI;

        searchField = new JSONStorableString("Search", "", OnSearchChanged);
    }

    public void InitUI()
    {
        backDrop = new GameObject("Backdrop");
        backDrop.transform.SetParent(script.UITransform, false);

        var rect = backDrop.AddComponent<RectTransform>();
        rect.anchorMin = Vector2.zero;
        rect.anchorMax = Vector2.one;
        rect.offsetMin = Coordinates.GET_BACKDROP_OFFSET_MIN();
        rect.offsetMax = Coordinates.GET_BACKDROP_OFFSET_MAX();

        var bg = backDrop.AddComponent<Image>();
        bg.color = new Color(0.4549019608f, 0.5960784314f, 0.8235294118f, 0.8f);

        var backDropTitle = new GameObject("Backdrop Title");
        backDropTitle.transform.SetParent(backDrop.transform, false);

        var rectTitle = backDropTitle.AddComponent<RectTransform>();
        rectTitle.anchorMin = Vector2.zero;
        rectTitle.anchorMax = Vector2.one;
        rectTitle.offsetMin = new Vector2(0, 1318);
        rectTitle.offsetMax = new Vector2(0, 0);

        var bgTitle = backDropTitle.AddComponent<Image>();
        bgTitle.color = new Color(0.1333333333f, 0.1960784314f, 0.3215686275f, 1);

        var backDropBottom = new GameObject("Backdrop Title");
        backDropBottom.transform.SetParent(backDrop.transform, false);

        var rectBottom = backDropBottom.AddComponent<RectTransform>();
        rectBottom.anchorMin = Vector2.zero;
        rectBottom.anchorMax = Vector2.one;
        rectBottom.offsetMin = new Vector2(0, 0);
        rectBottom.offsetMax = new Vector2(0, -1306);

        var bgTBottom = backDropBottom.AddComponent<Image>();
        bgTBottom.color = new Color(0.1764705882f, 0.2862745098f, 0.4470588235f, 1);

        scrollContentLeft = CreateScroll(backDrop.transform);
        scrollContentRight = CreateScroll(backDrop.transform, true);
    }

    override
    public void RefreshUI()
    {
        RemoveAllUI();
        UnityEngine.Object.Destroy(backDropLock);

        new TitleBuilder(script, uiElements, "Action Routing").FontSize(38).Parent(backDrop.transform).Location(new Vector2(20, -76)).Size(new Vector2(0, 100)).Build();

        new TextInputBuilder(script, uiElements, "Search:", searchField)
        .Parent(backDrop.transform).Location(new Vector2(620, -45)).Size(new Vector2(500, 40)).Build();

        new ButtonBuilder(script, uiElements, "X", () => { searchField.val = ""; })
        .Parent(backDrop.transform).Location(new Vector2(1125, -55)).Size(new Vector2(-1130, 50)).Build();

        var close = UIHelpers.CreateCustomButton(script, "<", Coordinates.GET_FOLD_LOCATION(), new Vector2(-1040, 1380));
        close.button.onClick.AddListener(() => { backDrop.gameObject.SetActive(!backDrop.gameObject.activeSelf); close.label = (close.label == "<") ? ">" : "<"; });
        uiElements.Add(close);

        new ButtonBuilder(script, uiElements, "+", () => ExpandAllGroups()).Parent(backDrop.transform).Location(new Vector2(500, -55)).Size(new Vector2(-1130, 50)).Build();
        new ButtonBuilder(script, uiElements, "-", () => CloseAllGroups()).Parent(backDrop.transform).Location(new Vector2(550, -55)).Size(new Vector2(-1130, 50)).Build();

        new TitleBuilder(script, uiElements, "Single click - add/remove link. Double click - execute action.\nWhite - normal actions. Yellow - layered actions. Green - transition actions")
        .FontSize(22).LabelColor(Color.white).Parent(backDrop.transform).Location(new Vector2(10, -1390)).Size(new Vector2(5, 100)).Build();

        if (actionsDictionary.Count > 1)
        {
            DrawRemainingActions();
            DrawLinkedActions();
        }

        if (script.lockedOnTransitionAction)
        {
            backDropLock = new GameObject("Backdrop Lock");
            backDropLock.transform.SetParent(backDrop.transform, false);

            var rectLock = backDropLock.AddComponent<RectTransform>();
            rectLock.anchorMin = Vector2.zero;
            rectLock.anchorMax = Vector2.one;
            rectLock.offsetMin = new Vector2(0, 0);
            rectLock.offsetMax = new Vector2(0, 0);

            var bgRect = backDropLock.AddComponent<Image>();
            bgRect.color = new Color(0, 0, 0, 0.7f);
        }
    }

    private RectTransform CreateScroll(Transform parent, bool rightSide = false)
    {
        var scrollView = new GameObject("ScrollView").AddComponent<RectTransform>();
        scrollView.SetParent(parent, false);

        var scrollRect = scrollView.gameObject.AddComponent<ScrollRect>();

        if (rightSide)
        {
            scrollView.anchorMin = new Vector2(0.51f, 0.05f);
            scrollView.anchorMax = new Vector2(0.99f, 0.95f);
        }
        else
        {
            scrollView.anchorMin = new Vector2(0.01f, 0.05f);
            scrollView.anchorMax = new Vector2(0.49f, 0.95f);
        }
        scrollView.offsetMin = Vector2.zero;
        scrollView.offsetMax = Vector2.zero;
        scrollView.gameObject.AddComponent<Mask>().showMaskGraphic = false;
        scrollView.gameObject.AddComponent<Image>(); // Opcjonalnie, dla tła

        var scrollContent = new GameObject("Content").AddComponent<RectTransform>();
        scrollContent.SetParent(scrollView.transform, false);
        scrollContent.pivot = new Vector2(0.5f, 1f);
        scrollContent.anchorMin = Vector2.zero;
        scrollContent.anchorMax = Vector2.one;
        scrollContent.offsetMin = Vector2.zero;
        scrollContent.offsetMax = Vector2.zero;
        var contentSizeFitter = scrollContent.gameObject.AddComponent<ContentSizeFitter>();
        contentSizeFitter.verticalFit = ContentSizeFitter.FitMode.PreferredSize;

        scrollRect.content = scrollContent;
        scrollRect.vertical = true;
        // scrollRect.horizontal = false; // Włącz, jeśli potrzebujesz poziomego przewijania

        var layoutGroup = scrollContent.gameObject.AddComponent<VerticalLayoutGroup>();
        layoutGroup.childControlHeight = true;
        layoutGroup.childForceExpandHeight = false;
        layoutGroup.childAlignment = TextAnchor.UpperLeft;
        layoutGroup.spacing = 10;

        return scrollContent;
    }

    // GROUP HANDLING

    private List<IGrouping<string, ActionItem>> GroupActions()
    {
        return actionsDictionary.Values
                .GroupBy(a => a.Group)
                .OrderBy(g => g.Key)
                .ToList();
    }

    private void DrawRemainingActions()
    {
        string currentSelectedAction = actionsChooser.val;
        ActionItem currentlySelectedActionItem = actionsDictionary[currentSelectedAction];

        new TitleBuilder(script, uiElements, "╔═══════  Remaining Actions  ═══════╗").LabelColor(Color.black).FontSize(24).FontBold().OtherStyle().Parent(scrollContentRight).Build();

        foreach (var group in GroupActions())
        {
            string groupName = group.Key;
            RefreshGroupExpansion(groupName);
            List<ActionItem> actionsInGroup = group.ToList();

            // Create or remove group button depending on whether the group is expanded.
            bool isGroupExpanded = !string.IsNullOrEmpty(searchField.val) || (groupExpanded.ContainsKey(groupName) && groupExpanded[groupName]);
            if (actionsInGroup.Any(a => !currentlySelectedActionItem.LinkedActions.Contains(a.Name) && a.Name != currentSelectedAction))
            {
                if (GroupOrAnyActionMatchesSearch(group))
                {
                    CreateGroupButtonAndActionButtons(groupName, false, scrollContentRight);
                }
            }
        }
    }

    private void DrawLinkedActions()
    {
        string currentSelectedAction = actionsChooser.val;
        ActionItem currentlySelectedActionItem = actionsDictionary[currentSelectedAction];
        new TitleBuilder(script, uiElements, "╔═════════  Linked Actions  ═════════╗").LabelColor(Color.black).FontSize(24).FontBold().OtherStyle().Parent(scrollContentLeft).Build();
        foreach (var group in GroupActions())
        {
            string groupName = group.Key;
            RefreshGroupExpansion(groupName);
            List<ActionItem> actionsInGroup = group.ToList();

            // Create or remove group button depending on whether the group is expanded.
            bool isGroupExpanded = groupExpanded.ContainsKey(groupName) && groupExpanded[groupName];
            if (actionsInGroup.Any(a => currentlySelectedActionItem.LinkedActions.Contains(a.Name) && a.Name != currentSelectedAction))
            {
                if (GroupOrAnyActionMatchesSearch(group))
                {
                    CreateGroupButtonAndActionButtons(groupName, true, scrollContentLeft);
                }
            }
        }
    }

    private bool GroupOrAnyActionMatchesSearch(IGrouping<string, ActionItem> group)
    {
        return GroupMatchesSearch(group.Key) || group.ToList().Any(actionItem => actionItem.Name.ToLower().Contains(searchField.val.ToLower()));
    }

    private bool GroupMatchesSearch(string groupName)
    {
        return groupName.ToLower().Contains(searchField.val.ToLower());
    }

    private void CreateGroupButtonAndActionButtons(string groupName, bool isLinked, RectTransform parent)
    {
        string currentSelectedAction = actionsChooser.val;
        UnityAction mainAction;
        if (isLinked)
        {
            mainAction = () => RemoveAllActionsFromGroup(groupName);
        }
        else
        {
            mainAction = () => AddAllActionsFromGroup(groupName, actionsChooser.val);
        }

        UnityAction toggleAction = () => ToggleGroup(groupName);

        bool isGroupExpanded = !string.IsNullOrEmpty(searchField.val) || (groupExpanded.ContainsKey(groupName) && groupExpanded[groupName]);

        new TwinButtonBuilder(script, uiElements,
            groupName != "" ? "Group: " + groupName : "Ungrouped",
            mainAction,
            isGroupExpanded ? "-" : "+",
            toggleAction).FirstWidth(0.9f).FirstStart(0).LabelBold(true).Parent(parent).Build();

        if (isGroupExpanded)
        {
            foreach (var actionItem in actionsDictionary.Values.Where(a => a.Group == groupName))
                if ((actionItem.Name != currentSelectedAction) &&
                (isLinked && actionsDictionary[currentSelectedAction].LinkedActions.Contains(actionItem.Name) || (!isLinked && !actionsDictionary[currentSelectedAction].LinkedActions.Contains(actionItem.Name))))
                {
                    if (GroupMatchesSearch(groupName) || actionItem.Name.ToLower().Contains(searchField.val.ToLower()))
                    {
                        CreateActionButton(actionItem, parent);
                    }
                }
        }
    }

    private void AddAllActionsFromGroup(string groupName, string currentSelectedAction)
    {
        if (!string.IsNullOrEmpty(currentSelectedAction))
        {
            ActionItem currentActionItem = actionsDictionary[currentSelectedAction];
            var actionsToAdd = actionsDictionary.Values
                .Where(a => a.Group == groupName && a.Name != currentSelectedAction)
                .Select(a => a.Name);

            foreach (var action in actionsToAdd)
            {
                if (!currentActionItem.LinkedActions.Contains(action))
                {
                    currentActionItem.LinkedActions.Add(action);
                }
            }

            RefreshUI();
        }
    }

    private void RemoveAllActionsFromGroup(string groupName)
    {
        string currentSelectedAction = actionsChooser.val;
        if (!string.IsNullOrEmpty(currentSelectedAction))
        {
            ActionItem currentActionItem = actionsDictionary[currentSelectedAction];
            var actionsToRemove = actionsDictionary.Values
                .Where(a => a.Group == groupName && a.Name != currentSelectedAction)
                .Select(a => a.Name);

            foreach (var action in actionsToRemove)
            {
                currentActionItem.LinkedActions.Remove(action);
            }

            RefreshUI();
        }
    }

    private void ToggleGroup(string groupName)
    {
        groupExpanded[groupName] = !groupExpanded[groupName];

        RefreshUI();
    }

    private void CreateActionButton(ActionItem actionItem, RectTransform parent)
    {
        new TwinButtonBuilder(script, uiElements,
        GetButtonName(actionItem),
        () => OnActionButtonClicked(actionItem.Name),
        "←",
        () => { actionsChooser.val = actionItem.Name; })
        .FirstWidth(0.9f).FirstStart(0.05f).Parent(parent)
        .ButtonColor(DefermineActionButtonColor(actionItem))
        .Build();
    }

    private Color DefermineActionButtonColor(ActionItem actionItem)
    {
        if (!string.IsNullOrEmpty(actionItem.Layer))
            return Colors.BUTTON_LAYERED_ACTION;

        if (actionItem.TransitionAction)
            return Colors.BUTTON_TRANSITION_ACTION;

        return Color.white;
    }

    private string GetButtonName(ActionItem actionItem)
    {
        return actionItem.Name + (string.IsNullOrEmpty(actionItem.Layer) ? "" : " (" + actionItem.Layer + ")");
    }

    private void OnActionButtonClicked(string name)
    {
        float timeSinceLastClick = Time.time - lastClickTime;
        if (timeSinceLastClick <= doubleClickThreshold)
        {
            OnActionButtonProcessDoubleClick(name);
            pendingSingleClickName = null;
        }
        else
        {
            pendingSingleClickName = name;
            script.StartCoroutine(WaitAndProcessSingleClick(name));
        }
        lastClickTime = Time.time;
    }

    private IEnumerator WaitAndProcessSingleClick(string name)
    {
        yield return new WaitForSeconds(doubleClickThreshold);

        if (pendingSingleClickName == name)
        {
            OnActionButtonProcessSingleClick(name);
        }
    }

    private void OnActionButtonProcessSingleClick(string name)
    {
        string currentSelectedAction = actionsChooser.val;
        if (string.IsNullOrEmpty(currentSelectedAction))
        {
            SuperController.LogError("No action is currently selected.");
            return;
        }

        ActionItem currentActionItem = actionsDictionary[currentSelectedAction];
        bool isLinked = currentActionItem.LinkedActions.Contains(name);
        if (isLinked)
        {
            currentActionItem.LinkedActions.Remove(name);
        }
        else
        {
            LinkActionItemToName(currentActionItem, name);
        }

        RefreshUI();
        linkedActionsUI.RefreshUI();
    }

    public void LinkActionItemToName(ActionItem actionItem, string nameToAdd)
    {
        if (!ActionItemHasSameActionName(actionItem, nameToAdd))
        {
            if (!string.IsNullOrEmpty(actionItem.Layer))
            {
                actionItem.Layer = "";
            }
            actionItem.LinkedActions.Add(nameToAdd);
        }
        else
        {
            SuperController.LogError("Action with function name = " + actionsDictionary[nameToAdd].Action + " already linked. Cannot have duplicate function names in the same route");
        }
    }

    private void OnActionButtonProcessDoubleClick(string name)
    {
        try
        {
            script.ExecuteAction(actionsDictionary[name]);
        }
        catch (Exception) { }
    }

    private bool ActionItemHasSameActionName(ActionItem actionItem, string nameToAdd)
    {
        foreach (string name in actionItem.LinkedActions)
        {
            if (actionsDictionary[name].Action == actionsDictionary[nameToAdd].Action)
            {
                return true;
            }
        }
        return false;
    }

    private void RefreshGroupExpansion(string groupName)
    {
        if (!groupExpanded.ContainsKey(groupName))
        {
            groupExpanded.Add(groupName, true);
        }
    }

    // GROUP HANDLING

    private void OnSearchChanged(string newSearch)
    {
        RefreshUI();
    }

    private void ExpandAllGroups()
    {
        groupExpanded = groupExpanded.ToDictionary(pair => pair.Key, pair => true);
        RefreshUI();
    }

    private void CloseAllGroups()
    {
        groupExpanded = groupExpanded.ToDictionary(pair => pair.Key, pair => false);
        RefreshUI();
    }
}
