using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class ButtonBuilder : UIBuilder<ButtonBuilder>
{
    private string label;
    private int fontSize = 30;
    private bool fontBold = true;
    private bool otherStyle = false;
    private UnityAction callback;
    private Vector2 location;
    private Vector2 size;

    public ButtonBuilder(MVRScript script, List<UIDynamic> uiElements, string label, UnityAction callback) : base(script, uiElements)
    {
        this.label = label;
        this.callback = callback;
    }
    public ButtonBuilder FontSize(int fontSize)
    {
        this.fontSize = fontSize;
        return this;
    }
    public ButtonBuilder FontBold()
    {
        fontBold = true;
        return this;
    }
    public ButtonBuilder Location(Vector2 location)
    {
        this.location = location;
        return this;
    }
    public ButtonBuilder Size(Vector2 size)
    {
        this.size = size;
        return this;
    }
    public UIDynamic Build()
    {
        var title = Utils.SetupButton(script, label, callback, rightSide);
        FinalizeBuild(title);
        var rectTransform = title.GetComponent<RectTransform>();
        if (location != null)
        {
            UnityEngine.Object.Destroy(title.GetComponent<LayoutElement>());
            rectTransform.pivot = Vector2.zero;
            rectTransform.anchoredPosition = location;
        }
        if (size != null)
        {
            rectTransform.sizeDelta = size;
        }

        return title;
    }
}
