using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class TitleBuilder : UIBuilder<TitleBuilder>
{
    private string text;
    private int fontSize = 30;
    private bool fontBold = true;
    private bool otherStyle = false;
    private Color labelColor = Color.white;
    private Vector2 location;
    private Vector2 size;
    public TitleBuilder(MVRScript script, List<UIDynamic> uiElements, string text) : base(script, uiElements)
    {
        this.text = text;
    }
    public TitleBuilder FontSize(int fontSize)
    {
        this.fontSize = fontSize;
        return this;
    }
    public TitleBuilder FontBold()
    {
        this.fontBold = true;
        return this;
    }
    public TitleBuilder OtherStyle()
    {
        this.otherStyle = true;
        return this;
    }
    public TitleBuilder LabelColor(Color labelColor)
    {
        this.labelColor = labelColor;
        return this;
    }
    public TitleBuilder Location(Vector2 location)
    {
        this.location = location;
        return this;
    }
    public TitleBuilder Size(Vector2 size)
    {
        this.size = size;
        return this;
    }
    public UIDynamic Build()
    {
        var title = Utils.CreateTitle(script, text, rightSide, fontSize, fontBold, otherStyle);
        title.gameObject.GetComponent<Text>().color = labelColor;
        FinalizeBuild(title);
        var rectTransform = title.GetComponent<RectTransform>();
        if (location != null)
        {
            rectTransform.pivot = Vector2.zero;
            rectTransform.anchoredPosition = location;
        }
        if (size != null)
        {
            rectTransform.sizeDelta = size;
        }

        return title;
    }
}
